import requests
from zephyr_sdk.exceptions.ZExceptions import ZAPIError


# Get all the currently logged in users.
def get_current_logged_in_users(client):
    url = client.base_url + '/user/current'
    headers = {
        'Authorization': 'Bearer ' + client.token
    }
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()



import requests
from zephyr_sdk.exceptions.ZExceptions import ZAPIError
from zephyr_sdk.exceptions.ZExceptions import MissingParametersError


# Create a requirement.
def create_requirement(client, requirement_spec):
    url = client.base_url + '/requirement/'
    headers = {
        'Authorization': 'Bearer ' + client.token,
        'Content-Type': 'application/json'
    }

    # Validate request body
    error_list = []
    if 'requirementTreeId' not in requirement_spec:
        error_list.append("requirementTreeId")
    if 'name' not in requirement_spec:
        error_list.append("name")

    if len(error_list) > 0:
        raise MissingParametersError(error_list)

    r = requests.post(url, headers=headers, json=requirement_spec)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()


# Delete a requirement by requirement id
def delete_requirement(client, req_id):
    url = client.base_url + '/requirement/' + str(req_id)
    headers = {
        'Authorization': 'Bearer ' + client.token,
    }
    r = requests.delete(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()


# Get a list of requirements from the requirement tree.
def get_requirements(client, req_tree_id):
    url = client.base_url + '/requirement?requirementtreeid=' + str(req_tree_id)
    headers = {
        'Authorization': 'Bearer ' + client.token,
    }
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()


# Update a requirement when given the id.
def update_requirement(client, req_id, requirement_spec):
    url = client.base_url + '/requirement/' + req_id
    headers = {
        'Authorization': 'Bearer ' + client.token,
        'Content-Type': 'application/json'
    }
    r = requests.put(url, headers=headers, json=requirement_spec)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()



class ZAPIError(Exception):
    def __init__(self, status_code, error_response):
        self.message = "ZAPI returned " + str(status_code) + ": " + error_response['errorMsg']
        self.status_code = status_code
        self.error_message = error_response['errorMsg']
        super().__init__(self.message)

    pass


class ResourceNotFoundError(Exception):
    def __init__(self, resource_type, resource_name):
        self.message = resource_type + " '" + resource_name + "' was not found."
        self.resource_type = resource_type
        self.resource_name = resource_name
        super().__init__(self.message)

    pass


class InsufficientContextError(Exception):
    def __init__(self, context_resource):
        self.message = "There was insufficient context: " + context_resource + " has not been set."
        self.missing_context = context_resource
        super().__init__(self.message)

    pass


class MissingParametersError(Exception):
    def __init__(self, parameters):
        self.message = "The following parameters were missing from the caller: " + ", ".join(parameters)
        self.missing_parameters = parameters
        super().__init__(self.message)

    pass


class MethodNotImplementedError(Exception):
    def __init__(self, message):
        self.message = "This method has not fully been implemented yet: " + message
        super().__init__(self.message)

    pass



import requests
from zephyr_sdk.exceptions.ZExceptions import ZAPIError
from zephyr_sdk.exceptions.ZExceptions import MissingParametersError


def create_defect(client, defect_spec):
    url = client.base_url + '/defect'
    headers = {
        "Authorization": "Bearer " + client.token,
        "Content-Type": "application/json"
    }

    # Validate the request body
    error_list = []
    if 'projectId' not in defect_spec:
        error_list.append("projectId")
    if 'product' not in defect_spec:
        error_list.append("product")

    if len(error_list) > 0:
        raise MissingParametersError(error_list)

    r = requests.post(url, headers=headers, json=defect_spec)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()



import requests
from zephyr_sdk.exceptions.ZExceptions import ZAPIError
from zephyr_sdk.exceptions.ZExceptions import MissingParametersError


def create_testcase(client, testcase_spec):
    url = client.base_url + '/testcase'
    headers = {
        "Authorization": "Bearer " + client.token,
        "Content-Type": "application/json"
    }

    # Validate request body
    error_list = []
    if 'name' not in testcase_spec['testcase']:
        error_list.append("testcase.name")
    if 'tcrCatalogTreeId' not in testcase_spec:
        error_list.append("tcrCatalogTreeId")

    if len(error_list) > 0:
        raise MissingParametersError(error_list)

    r = requests.post(url, headers=headers, json=testcase_spec)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()


def map_testcase_to_requirements(client, map_spec):
    url = client.base_url + "/testcase/allocate/requirement"
    headers = {
        "Content-Type": "application/json",
        "Authorization": "Bearer " + client.token
    }

    r = requests.post(url, headers=headers, json=map_spec)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())
    return r.json()



import requests
from zephyr_sdk.exceptions.ZExceptions import ZAPIError


def get_testcase_tree_by_release_id(client, release_id):
    url = client.base_url + '/testcasetree/phases/' + str(release_id)
    headers = {
        "Authorization": "Bearer " + client.token
    }

    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())
    return r.json()



import requests
from zephyr_sdk.exceptions.ZExceptions import ZAPIError
from zephyr_sdk.exceptions.ZExceptions import MissingParametersError


# Create requirement tree
def create_requirement_tree(client, req_tree_spec):
    url = client.base_url + '/requirementtree/add'
    headers = {
        'Authorization': 'Bearer ' + client.token,
        'Content-Type': 'application/json'
    }

    # Validate request body
    error_list = []
    if 'projectId' not in req_tree_spec:
        error_list.append("projectId")

    if 'name' not in req_tree_spec:
        error_list.append("name")

    if 'description' not in req_tree_spec:
        error_list.append("description")

    if len(error_list) > 0:
        raise MissingParametersError(error_list)

    r = requests.post(url, headers=headers, json=req_tree_spec)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()


# Get information on all requirement trees
def get_all_requirement_trees(client):
    url = client.base_url + '/requirementtree'
    headers = {
        'Authorization': 'Bearer ' + client.token
    }
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()


# Get the requirement trees in a specific project
def get_requirement_tree_with_ids(client, project_id, release_id):
    url = client.base_url + '/requirementtree?projectId=' + str(project_id)

    if release_id is not None:
        url += '&releaseid=' + str(release_id)

    headers = {
        'Authorization': 'Bearer ' + client.token
    }
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()




import requests
from zephyr_sdk.exceptions.ZExceptions import ZAPIError


# Get the details of all projects.
def get_all_normal_projects_details(client):
    url = client.base_url + '/project/details'
    headers = {
        'Authorization': 'Bearer ' + client.token
    }
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()


# Gets the id of all projects, and returns them in an array.
def get_all_normal_project(client):
    url = client.base_url + '/project/normal'
    headers = {
        'Authorization': 'Bearer ' + client.token
    }
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()


# Gets the id of every project lead.
def get_lead_for_all_projects(client):
    url = client.base_url + '/project/all/leads'
    headers = {
        'Authorization': 'Bearer ' + client.token
    }
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()


# Gets project details by the id passed in.
def get_project_by_id(client, project_id):
    url = client.base_url + '/project/' + str(project_id)
    headers = {
        'Authorization': 'Bearer ' + client.token
    }
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()


# Gets the team count for all projects
def get_project_team_count_for_all_projects(client):
    url = client.base_url + '/project/count/allprojects'
    headers = {
        'Authorization': 'Bearer ' + client.token
    }
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()


# Gets the number of projects each user is a member of.
def get_project_team_count_for_all_users(client):
    url = client.base_url + '/project/count/allusers'
    headers = {
        'Authorization': 'Bearer ' + client.token
    }
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()


# Get the user ids for the teams for all projects
def get_project_team_for_allocated_projects(client):
    url = client.base_url + '/project/allocated/projects'
    headers = {
        'Authorization': 'Bearer ' + client.token
    }
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()


# Get the user ids of the members on a specific project
def get_project_team_for_project(client, project_id):
    url = client.base_url + '/project/projectteam/' + str(project_id)
    headers = {
        'Authorization': 'Bearer ' + client.token
    }
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()


# Get all projects
def get_all_projects(client, include_inactive):
    url = client.base_url + '/project?includeinactive=' + str(include_inactive)
    headers = {
        'Authorization': 'Bearer ' + client.token
    }
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()


# Light call to get all projects. Only ids and names
def get_all_projects_lite(client):
    url = client.base_url + '/project/lite'
    headers = {
        'Authorization': 'Bearer ' + client.token
    }
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()



from zephyr_sdk.components import defects
from zephyr_sdk.components import projects
from zephyr_sdk.components import releases
from zephyr_sdk.components import requirements
from zephyr_sdk.components import requirementtrees
from zephyr_sdk.components import testcases
from zephyr_sdk.components import testcasetrees
from zephyr_sdk.components import users
from zephyr_sdk.exceptions.ZExceptions import ResourceNotFoundError
from zephyr_sdk.exceptions.ZExceptions import InsufficientContextError
from zephyr_sdk.exceptions.ZExceptions import MethodNotImplementedError


class ZClient:
    # Constructor for the zephyr client.
    def __init__(self, token, url):
        self.token = token
        self.base_url = url

        # Set user id for the API calls
        self.user_id = self.get_current_logged_in_users()['id']

        # Properties that are undeclared for now
        self.project_id = None
        self.project_name = None
        self.release_id = None
        self.req_tree_id = None
        self.testcase_tree_id = None

    # Methods to add context to the client itself.
    def set_project(self, project_name):
        # Find the project with the name passed in
        response = self.get_all_projects_lite()
        for project in response:
            # Set the project id and exit function if found
            if project['name'] == project_name:
                self.project_id = project['id']
                self.project_name = project_name
                return

        # If the method got to this point, than raise an exception.
        raise ResourceNotFoundError("Project", project_name)

    def set_release(self, release_name):
        # First check to see if the project was set
        if self.project_id is None:
            raise InsufficientContextError("Project")

        # Now find the release with the name passed in
        response = self.get_releases_for_a_project(self.project_id)
        for release in response:
            # Set the release id and exit the function if found
            if release['name'] == release_name:
                self.release_id = release['id']
                return

        # If the method got to this point, raise an exception
        raise ResourceNotFoundError("Release", release_name)

    def set_requirement_tree(self, tree_name):
        # First check to see if the project was set
        if self.project_id is None:
            raise InsufficientContextError("Project")

        # Now find the requirement tree with the specified name.
        response = self.get_requirement_tree_with_ids(self.project_id, self.release_id)
        for tree in response:
            # Set the requirement tree id to be the proper tree if found.
            if tree['name'] == tree_name:
                self.req_tree_id = tree['id']
                return

        # Raise an exception that tree was not found.
        raise ResourceNotFoundError("Requirement tree", tree_name)

    def set_testcase_tree(self, tree_name):
        # First check to see if release id has been set.
        if self.release_id is None:
            raise InsufficientContextError("Release")

        # Now make the API call to get the testcase tree ids
        response = self.get_testcase_tree_by_release_id()
        for tree in response:
            if tree['name'] == tree_name:
                self.testcase_tree_id = tree['id']
                return

        raise ResourceNotFoundError("Testcase Tree", tree_name)

    # Defects methods
    def create_defect(self, defect_spec):
        if 'projectId' not in defect_spec and self.project_id is not None:
            defect_spec['projectId'] = self.project_id
        if 'product' not in defect_spec and self.project_name is not None:
            defect_spec['product'] = self.project_name
        return defects.create_defect(self, defect_spec)

    # Projects methods
    def get_all_normal_projects_details(self):
        return projects.get_all_normal_projects_details(self)

    def get_all_normal_project(self):
        return projects.get_all_normal_project(self)

    def get_lead_for_all_projects(self):
        return projects.get_lead_for_all_projects(self)

    def get_project_by_id(self, project_id=None):
        if project_id is None:
            return projects.get_project_by_id(self, self.project_id)
        else:
            return projects.get_project_by_id(self, project_id)

    def get_project_team_count_for_all_projects(self):
        return projects.get_project_team_count_for_all_projects(self)

    def get_project_team_count_for_all_users(self):
        return projects.get_project_team_count_for_all_users(self)

    def get_project_team_for_allocated_projects(self):
        return projects.get_project_team_for_allocated_projects(self)

    def get_project_team_for_project(self, project_id=None):
        raise MethodNotImplementedError("This method produces too much output.")
        if project_id is None:
            return projects.get_project_team_for_project(self, self.project_id)
        else:
            return projects.get_project_team_for_project(self, project_id)

    def get_all_projects(self, include_inactive):
        raise MethodNotImplementedError("Response always returns empty.")
        return projects.get_all_projects(self, include_inactive)

    def get_all_projects_lite(self):
        return projects.get_all_projects_lite(self)

    # Releases methods
    def create_release(self, release_spec):
        if 'projectId' not in release_spec and self.project_id is not None:
            release_spec['projectId'] = self.project_id
        return releases.create_release(self, release_spec)

    def delete_release(self, release_id):
        return releases.delete_release(self, release_id)

    def get_release_by_release_id(self, release_id=None):
        if release_id is None:
            return releases.get_release_by_release_id(self, self.release_id)
        else:
            return releases.get_release_by_release_id(self, release_id)

    def get_releases_for_a_project(self, project_id=None):
        if project_id is None:
            return releases.get_releases_for_a_project(self, self.project_id)
        else:
            return releases.get_releases_for_a_project(self, project_id)

    # Requirements methods
    def create_requirement(self, requirement_spec):
        if 'requirementTreeId' not in requirement_spec and self.req_tree_id is not None:
            requirement_spec['requirementTreeId'] = self.req_tree_id
        return requirements.create_requirement(self, requirement_spec)

    def delete_requirement(self, req_id):
        return requirements.delete_requirement(self, req_id)

    def get_requirements(self, req_tree_id=None):
        if req_tree_id is None:
            return requirements.get_requirements(self, self.req_tree_id)
        else:
            return requirements.get_requirements(self, req_tree_id)

    def update_requirement(self, req_id, requirement_spec):
        return requirements.update_requirement(self, req_id, requirement_spec)

    # RequirementTrees methods
    def create_requirement_tree(self, req_tree_spec):
        if 'projectId' not in req_tree_spec and self.project_id is not None:
            req_tree_spec['projectId'] = self.project_id
        if 'releaseIds' not in req_tree_spec and self.release_id is not None:
            req_tree_spec['releaseIds'] = [self.release_id]
        if 'parentId' not in req_tree_spec and self.req_tree_id is not None:
            req_tree_spec['parentId'] = self.req_tree_id

        return requirementtrees.create_requirement_tree(self, req_tree_spec)

    def get_all_requirement_trees(self):
        return requirementtrees.get_all_requirement_trees(self)

    def get_requirement_tree_with_ids(self, project_id=None, release_id=None):
        if project_id is None:
            project_id = self.project_id
        if release_id is None:
            release_id = self.release_id

        return requirementtrees.get_requirement_tree_with_ids(self, project_id, release_id)

    # Testcases methods
    def create_testcase(self, testcase_spec):
        if 'testcase' not in testcase_spec:
            testcase_spec['testcase'] = {}
        if 'releaseId' not in testcase_spec['testcase'] and self.release_id is not None:
            testcase_spec['testcase']['releaseId'] = self.release_id
        if 'tcrCatalogTreeId' not in testcase_spec and self.testcase_tree_id is not None:
            testcase_spec['tcrCatalogTreeId'] = self.testcase_tree_id

        return testcases.create_testcase(self, testcase_spec)

    def map_testcase_to_requirements(self, map_spec):
        return testcases.map_testcase_to_requirements(self, map_spec)

    # Testcase tree methods
    def get_testcase_tree_by_release_id(self, release_id=None):
        if release_id is None:
            release_id = self.release_id
        return testcasetrees.get_testcase_tree_by_release_id(self, release_id)

    # Users methods
    def get_current_logged_in_users(self):
        return users.get_current_logged_in_users(self)



import requests
from zephyr_sdk.exceptions.ZExceptions import ZAPIError
from zephyr_sdk.exceptions.ZExceptions import MissingParametersError


# Create a release with the information passed in
def create_release(client, release_spec):
    url = client.base_url + '/release/'
    headers = {
        'Authorization': 'Bearer ' + client.token,
        'Content-Type': 'application/json'
    }

    # Validate the release_spec variable
    error_list = []
    if 'name' not in release_spec:
        error_list.append("name")

    if 'releaseStartDate' not in release_spec:
        error_list.append("releaseStartDate")

    if 'releaseEndDate' not in release_spec:
        error_list.append("releaseEndDate")

    if 'projectId' not in release_spec:
        error_list.append("projectId")

    # If any errors were added to the list, raise an exception
    if len(error_list) > 0:
        raise MissingParametersError(error_list)

    # Otherwise make the API call.
    r = requests.post(url, json=release_spec, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()


# Delete a release by release id.
def delete_release(client, release_id):
    url = client.base_url + '/release/' + str(release_id)
    headers = {
        'Authorization': 'Bearer ' + client.token
    }
    r = requests.delete(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()


# Get details of a release by the release id
def get_release_by_release_id(client, release_id):
    url = client.base_url + '/release/' + str(release_id)
    headers = {
        'Authorization': 'Bearer ' + client.token
    }
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()


# Get all releases for a project
def get_releases_for_a_project(client, project_id):
    url = client.base_url + '/release/project/' + str(project_id)
    headers = {
        'Authorization': 'Bearer ' + client.token
    }
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        raise ZAPIError(r.status_code, r.json())

    return r.json()
